@@ -47,7 +47,12 @@ class MultiheadAttention(nn.Module):
             nn.init.constant_(self.k_proj.bias, 0.)
         if self.v_proj.bias is not None:
             nn.init.constant_(self.v_proj.bias, 0.)
-        
+    
+    def _apply(self, fn):
+        super()._apply(fn)
+        self.device = next(self.parameters(), torch.empty(0)).device
+        return self
+    
     def forward(self, x, causal=False, rope=None):
         if self.batch_first:
             x = x.transpose(0, 1)
@@ -123,6 +128,11 @@ class LinearAttention(nn.Module):
         if self.v_proj.bias is not None:
             nn.init.constant_(self.v_proj.bias, 0.)
     
+    def _apply(self, fn):
+        super()._apply(fn)
+        self.device = next(self.parameters(), torch.empty(0)).device
+        return self
+    
     def forward(self, x, rope=None, causal=True):
         if self.batch_first:
             x = x.transpose(0, 1)
@@ -202,6 +212,11 @@ class OrthoLinearAttention(nn.Module):
         if self.v_proj.bias is not None:
             nn.init.constant_(self.v_proj.bias, 0.)
     
+    def _apply(self, fn):
+        super()._apply(fn)
+        self.device = next(self.parameters(), torch.empty(0)).device
+        return self
+    
     def forward(self, x, rope=None, causal=True):
         if self.batch_first:
             x = x.transpose(0, 1)
@@ -288,6 +303,11 @@ class CompressionAttention(nn.Module):
         if self.v_proj.bias is not None:
             nn.init.constant_(self.v_proj.bias, 0.)
     
+    def _apply(self, fn):
+        super()._apply(fn)
+        self.device = next(self.parameters(), torch.empty(0)).device
+        return self
+    
     def forward(self, x, rope=None, causal=True):
         if self.batch_first:
             x = x.transpose(0, 1)
@@ -395,14 +415,19 @@ class SlidingWindowAttention(nn.Module):
         if self.v_proj.bias is not None:
             nn.init.constant_(self.v_proj.bias, 0.)
     
+    def _apply(self, fn):
+        super()._apply(fn)
+        self.device = next(self.parameters(), torch.empty(0)).device
+        return self
+    
     @lru_cache(maxsize=2)
-    def causal_windowed_mask(self, seq_len, window_len, dilation=1, to_bias=False):
-        idxs = torch.arange(seq_len, device=self.device)
+    def causal_windowed_mask(self, seq_len, window_len, dilation=1, to_bias=False, device="cpu"):
+        idxs = torch.arange(seq_len, device=device)
         rows = idxs.unsqueeze(1)
         cols = idxs.unsqueeze(0)
         diff = rows - cols
 
-        allowed = (diff >= 0) & (diff <= dilation * (window_len - 1)) & ((diff % dilation) == 0)
+        allowed = (diff >= 0) & (diff // dilation >= 0) & (diff // dilation < window_len) & (diff % dilation == 0)
         allowed = allowed.unsqueeze(0)
         
         if not to_bias:
@@ -412,8 +437,8 @@ class SlidingWindowAttention(nn.Module):
         return mask
     
     @lru_cache(maxsize=2)
-    def symmetric_windowed_mask(self, seq_len, window_len, dilation=1, to_bias=False):
-        idxs = torch.arange(seq_len, device=self.device)
+    def symmetric_windowed_mask(self, seq_len, window_len, dilation=1, to_bias=False, device="cpu"):
+        idxs = torch.arange(seq_len, device=device)
         rows = idxs.unsqueeze(1)
         cols = idxs.unsqueeze(0)
 
@@ -421,7 +446,7 @@ class SlidingWindowAttention(nn.Module):
         diff = rows - cols
         abs_diff = diff.abs()
 
-        allowed = (abs_diff <= dilation * half) & ((diff % dilation) == 0)
+        allowed = (abs_diff // dilation <= half) & (diff % dilation == 0)
         allowed = allowed.unsqueeze(0)
         
         if not to_bias:
@@ -431,7 +456,7 @@ class SlidingWindowAttention(nn.Module):
         return mask
     
     @lru_cache(maxsize=2)
-    def causal_windowed_block_mask(self, src_len, tgt_len=None):
+    def causal_windowed_block_mask(self, src_len, tgt_len=None, device="cpu"):
         if tgt_len is None: tgt_len = src_len
         
         half_span = self.dilation * (self.window_len - 1)
@@ -440,10 +465,10 @@ class SlidingWindowAttention(nn.Module):
             diff = q_idx - kv_idx
             return (diff >= 0) & (diff <= half_span) & ((diff % self.dilation) == 0)
 
-        return create_block_mask(mask_mod, B=None, H=None, Q_LEN=src_len, KV_LEN=tgt_len, BLOCK_SIZE=1, device=self.device)
+        return create_block_mask(mask_mod, B=None, H=None, Q_LEN=src_len, KV_LEN=tgt_len, BLOCK_SIZE=1, device=device)
     
     @lru_cache(maxsize=2)
-    def symmetric_windowed_block_mask(self, src_len, tgt_len=None):
+    def symmetric_windowed_block_mask(self, src_len, tgt_len=None, device="cpu"):
         if tgt_len is None: tgt_len = src_len
         
         half = self.window_len // 2
@@ -453,7 +478,7 @@ class SlidingWindowAttention(nn.Module):
             diff = q_idx - kv_idx
             return diff.abs().le(half_span) & ((diff % self.dilation) == 0)
 
-        return create_block_mask(mask_mod,  B=None, H=None, Q_LEN=src_len, KV_LEN=tgt_len, BLOCK_SIZE=1, device=self.device)
+        return create_block_mask(mask_mod,  B=None, H=None, Q_LEN=src_len, KV_LEN=tgt_len, BLOCK_SIZE=1, device=device)
     
     def forward(self, x, rope=None, causal=True):
         if self.batch_first:
@@ -486,18 +511,18 @@ class SlidingWindowAttention(nn.Module):
             v = v.flatten(0, 1)
             
             if causal:
-                attn_mask = self.causal_windowed_mask(src_len, self.window_len, dilation=self.dilation, to_bias=True)  # (1, tgt_len, src_len)
+                attn_mask = self.causal_windowed_mask(src_len, self.window_len, dilation=self.dilation, to_bias=True, device=self.device)  # (1, tgt_len, src_len)
             else:
-                attn_mask = self.symmetric_windowed_mask(src_len, self.window_len, dilation=self.dilation, to_bias=True)  # (1, tgt_len, src_len)
+                attn_mask = self.symmetric_windowed_mask(src_len, self.window_len, dilation=self.dilation, to_bias=True, device=self.device)  # (1, tgt_len, src_len)
 
             attn_output = F.scaled_dot_product_attention(q, k, v, attn_mask=attn_mask, scale=1.0, dropout_p=self.dropout)  # (bsz * n_heads, tgt_len, d_head)
             
             attn_output = rearrange(attn_output, '(b h) s d -> s b (h d)', h=self.n_heads)
         else:
             if causal:
-                block_mask = self.causal_windowed_block_mask(src_len, tgt_len)
+                block_mask = self.causal_windowed_block_mask(src_len, tgt_len, device=self.device)
             else:
-                block_mask = self.symmetric_windowed_block_mask(src_len, tgt_len)
+                block_mask = self.symmetric_windowed_block_mask(src_len, tgt_len, device=self.device)
             
             attn_device = "cuda" if self.device == "cuda" else "cpu"
             
