@@ -501,7 +501,7 @@ class FlowTransformer(nn.Module):
         if self.abs_pos_encoding is not None:
             pos = torch.arange(seq_len, device=x.device, dtype=torch.long).unsqueeze(0).expand(x.size(0), -1)
             x = x + self.abs_pos_encoding(pos)
-        x = odeint_adjoint(self.internal_pass, x, torch.tensor([0., 1.], device=x.device), method='rk4', rtol=1e-3, atol=1e-4)[-1]
+        x = odeint(self.internal_pass, x, torch.tensor([0., 1.], device=x.device), method='rk4', rtol=1e-3, atol=1e-4)[-1]
         x = self.norm_f(x)
         x = self.out_proj(x)
         return x
\ No newline at end of file
